diff --git a/common/common.c b/common/common.c
index c0a56e3..6f8c9b0 100644
--- a/common/common.c
+++ b/common/common.c
@@ -75,6 +75,7 @@ void    x264_param_default( x264_param_t *param )
     param->i_bframe_adaptive = X264_B_ADAPT_FAST;
     param->i_bframe_bias = 0;
     param->b_bframe_pyramid = 0;
+    param->b_bstrict_hierarchical = 0;
 
     param->b_deblocking_filter = 1;
     param->i_deblocking_filter_alphac0 = 0;
@@ -356,6 +357,8 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         p->i_bframe_bias = atoi(value);
     OPT("b-pyramid")
         p->b_bframe_pyramid = atobool(value);
+    OPT("b-strict-hierarchical")
+        p->b_bstrict_hierarchical = atobool(value);
     OPT("nf")
         p->b_deblocking_filter = !atobool(value);
     OPT2("filter", "deblock")
@@ -873,6 +876,7 @@ char *x264_param2string( x264_param_t *p, int b_res )
     s += sprintf( s, " bframes=%d", p->i_bframe );
     if( p->i_bframe )
     {
+        s += sprintf( s, " b_strict_hierarchical=%d", p->b_bstrict_hierarchical );
         s += sprintf( s, " b_pyramid=%d b_adapt=%d b_bias=%d direct=%d wpredb=%d",
                       p->b_bframe_pyramid, p->i_bframe_adaptive, p->i_bframe_bias,
                       p->analyse.i_direct_mv_pred, p->analyse.b_weighted_bipred );
diff --git a/common/common.h b/common/common.h
index 574040d..9f37124 100644
--- a/common/common.h
+++ b/common/common.h
@@ -220,11 +220,18 @@ typedef struct
 
     int b_ref_pic_list_reordering_l0;
     int b_ref_pic_list_reordering_l1;
-    struct {
+    struct
+    {
         int idc;
         int arg;
     } ref_pic_list_order[2][16];
 
+    int i_mmco_idx;
+    struct // struct for future expansion
+    {
+        int i_difference_of_pic_nums;
+    } mmco[16];
+
     int i_cabac_init_idc;
 
     int i_qp;
@@ -385,6 +392,11 @@ struct x264_t
     x264_frame_t    *fref1[16+3];     /* ref list 1 */
     int             b_ref_reorder[2];
 
+    int             i_mmco_idx;
+    struct
+    {
+        int         i_difference_of_pic_nums;
+    } mmco[16];
 
 
     /* Current MB DCT coeffs */
diff --git a/encoder/encoder.c b/encoder/encoder.c
index af62765..0457c88 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -122,6 +122,14 @@ static void x264_slice_header_init( x264_t *h, x264_slice_header_t *sh,
 
     sh->b_ref_pic_list_reordering_l0 = h->b_ref_reorder[0];
     sh->b_ref_pic_list_reordering_l1 = h->b_ref_reorder[1];
+    
+    sh->i_mmco_idx = h->i_mmco_idx;
+    if( h->i_mmco_idx )
+    {
+        int j = 0;
+        for( ; j < h->i_mmco_idx; j++ )
+            sh->mmco[j].i_difference_of_pic_nums = h->mmco[j].i_difference_of_pic_nums;
+    }
 
     /* If the ref list isn't in the default order, construct reordering header */
     /* List1 reordering isn't needed yet */
@@ -139,6 +147,21 @@ static void x264_slice_header_init( x264_t *h, x264_slice_header_t *sh,
         }
     }
 
+    if( sh->b_ref_pic_list_reordering_l1 )
+    {
+        int pred_frame_num = i_frame;
+        for( i = 0; i < h->i_ref1; i++ )
+        {
+            
+            int diff = h->fref1[i]->i_frame_num - pred_frame_num;
+            if( diff == 0 )
+                x264_log( h, X264_LOG_ERROR, "diff frame num == 0\n" );
+            sh->ref_pic_list_order[1][i].idc = ( diff > 0 );
+            sh->ref_pic_list_order[1][i].arg = abs( diff ) - 1;
+            pred_frame_num = h->fref1[i]->i_frame_num;
+        }
+    }
+
     sh->i_cabac_init_idc = param->i_cabac_init_idc;
 
     sh->i_qp = i_qp;
@@ -272,7 +295,17 @@ static void x264_slice_header_write( bs_t *s, x264_slice_header_t *sh, int i_nal
         }
         else
         {
-            bs_write1( s, 0 );  /* adaptive_ref_pic_marking_mode_flag */
+            bs_write1( s, sh->i_mmco_idx > 0 ); /* adaptive_ref_pic_marking_mode_flag */
+            if( sh->i_mmco_idx > 0 )
+            {
+                int i = 0;
+                for( ; i < sh->i_mmco_idx; i++ )
+                {
+                    bs_write_ue( s, 1 ); /* mark short term ref as unused */
+                    bs_write_ue( s, sh->mmco[i].i_difference_of_pic_nums - 1 );
+                }
+                bs_write_ue( s, 0 );
+            }
         }
     }
 
@@ -1094,6 +1127,12 @@ static inline void x264_reference_build_list( x264_t *h, int i_poc )
                 break;
             }
     }
+    if( h->param.b_bframe_pyramid && h->sh.i_type != SLICE_TYPE_I )
+    {
+        h->b_ref_reorder[0] = 1;
+        if( h->sh.i_type == SLICE_TYPE_B )
+            h->b_ref_reorder[1] = 1;
+    }
 
     h->i_ref1 = X264_MIN( h->i_ref1, h->frames.i_max_ref1 );
     h->i_ref0 = X264_MIN( h->i_ref0, h->frames.i_max_ref0 );
@@ -1227,6 +1266,43 @@ static inline void x264_reference_reset( x264_t *h )
     h->fenc->i_poc = 0;
 }
 
+static inline void x264_reference_partial_reset( x264_t *h )
+{
+    int refs = 0;
+    int lastnonbidx = -1;
+    while( h->frames.reference[refs] )
+    {
+        if( h->frames.reference[refs]->i_type == X264_TYPE_I
+         || h->frames.reference[refs]->i_type == X264_TYPE_IDR
+         || h->frames.reference[refs]->i_type == X264_TYPE_P )
+            lastnonbidx = refs;
+        refs++;
+    }
+    assert( lastnonbidx > -1 );
+    // Remove any brefs from dpb remembering if current frame is bref
+    // x264 has not added it yet. only will be true for i or P frames.
+    while( refs > lastnonbidx+1 && h->param.b_bstrict_hierarchical
+                                && h->param.b_bframe_pyramid )
+    {
+        int diff = h->i_frame_num - h->frames.reference[refs-1]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_pop( h->frames.reference ) );
+        refs--;
+    }
+    // This makes room in the dpb for the delayed display time of the later b-frame's
+    // the x264_max is to make sure that a foward and backward reference is always left
+    // for b-frames incase too few reference frames are used with b-pyramid. ( < 4 )
+    while( ( refs + 2 ) > X264_MAX( h->frames.i_max_dpb, 4 )
+                                 && h->param.b_bframe_pyramid )
+    {
+        int diff = h->i_frame_num - h->frames.reference[0]->i_frame_num;
+        h->mmco[ h->i_mmco_idx++ ].i_difference_of_pic_nums = diff;
+        x264_frame_push_unused( h, x264_frame_shift( h->frames.reference ) );
+        lastnonbidx--;
+        refs--;
+    }
+}
+
 static inline void x264_slice_init( x264_t *h, int i_nal_type, int i_global_qp )
 {
     /* ------------------------ Create slice header  ----------------------- */
@@ -1705,6 +1781,7 @@ int     x264_encoder_encode( x264_t *h,
     {
         h->frames.i_last_idr = h->fenc->i_frame;
     }
+    h->i_mmco_idx = 0;
 
     /* ------------------- Setup frame context ----------------------------- */
     /* 5: Init data dependent of frame type */
@@ -1722,18 +1799,21 @@ int     x264_encoder_encode( x264_t *h,
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
         h->sh.i_type = SLICE_TYPE_I;
+        x264_reference_partial_reset( h );
     }
     else if( h->fenc->i_type == X264_TYPE_P )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* Not completely true but for now it is (as all I/P are kept as ref)*/
         h->sh.i_type = SLICE_TYPE_P;
+        x264_reference_partial_reset( h );
     }
     else if( h->fenc->i_type == X264_TYPE_BREF )
     {
         i_nal_type    = NAL_SLICE;
         i_nal_ref_idc = NAL_PRIORITY_HIGH; /* maybe add MMCO to forget it? -> low */
         h->sh.i_type = SLICE_TYPE_B;
+        x264_reference_partial_reset( h );
     }
     else    /* B frame */
     {
diff --git a/x264.c b/x264.c
index b7083c0..787db5f 100644
--- a/x264.c
+++ b/x264.c
@@ -191,6 +191,7 @@ static void Help( x264_param_t *defaults, int b_longhelp )
         "                                  - 2: Optimal (slow with high --bframes)\n", defaults->i_bframe_adaptive );
     H1( "      --b-bias <integer>      Influences how often B-frames are used [%d]\n", defaults->i_bframe_bias );
     H0( "      --b-pyramid             Keep some B-frames as references\n" );
+    H0( "      --b-strict-hierarchical Remove BREFs from dpb so no non B-frame refers to them\n" );
     H0( "      --no-cabac              Disable CABAC\n" );
     H0( "  -r, --ref <integer>         Number of reference frames [%d]\n", defaults->i_frame_reference );
     H1( "      --no-deblock            Disable loop filter\n" );
@@ -394,6 +395,8 @@ static struct option long_options[] =
     { "no-b-adapt",        no_argument, NULL, 0 },
     { "b-bias",      required_argument, NULL, 0 },
     { "b-pyramid",         no_argument, NULL, 0 },
+    { "b-strict-hierarchical",
+                           no_argument, NULL, 0 },
     { "min-keyint",  required_argument, NULL, 'i' },
     { "keyint",      required_argument, NULL, 'I' },
     { "scenecut",    required_argument, NULL, 0 },
diff --git a/x264.h b/x264.h
index 6609a32..c2b4f04 100644
--- a/x264.h
+++ b/x264.h
@@ -195,6 +195,7 @@ typedef struct x264_param_t
     int         i_bframe_adaptive;
     int         i_bframe_bias;
     int         b_bframe_pyramid;   /* Keep some B-frames as references */
+    int         b_bstrict_hierarchical; /* MMCO BREFs so only bframes from its group refer to them */
 
     int         b_deblocking_filter;
     int         i_deblocking_filter_alphac0;    /* [-6, 6] -6 light filter, 6 strong */